/***************************************************************************
 *   Copyright (C) 2011 by Rene Landert <rene.landert@bluewin.ch>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 **************************************************************************/

#include "KchildlockDaemon.h"
#include <KAboutData>
#include <KPluginFactory>
#include <klocalizedstring.h>
#include <KComponentData>
#include "../common/kchildlockcommon.h"
#include "systray_status.h"
#include <KUserGroup>


K_PLUGIN_FACTORY(KchildlockFactory,
                 registerPlugin<KchildlockDaemon>();)
K_EXPORT_PLUGIN(KchildlockFactory("kchildlockdaemon"))


// the main program 
//=================

KchildlockDaemon::KchildlockDaemon(QObject *parent, const QList<QVariant>&)
        : KDEDModule(parent)
{
  
    QDate currdate;
    QTime currtime;
  
    KAboutData aboutData("kchildlockdaemon", "kchildlockdaemon", ki18n("Kchildlock Daemon"),
                         KCHILDLOCK_VERSION, ki18n("A Daemon for Kchildlock for KDE4"),
                         KAboutData::License_GPL, ki18n("(c), 2015 Rene Landert"),
                         KLocalizedString(), "This daemon "
                             "restricts the usage time of the computer per day and "
                             "per week for selected users.");

    aboutData.addAuthor(ki18n("Rene Landert"), ki18n("Maintainer"),  "rene.landert@bluewin.ch", "http://www.sourceforge.net/kchildlock");
    KGlobal::locale()->insertCatalog("kchildlock");
    
    KComponentData applicationData;
    applicationData = KComponentData(aboutData);
 

    notificationTimer = new QTimer(this);
    startupTimer = new QTimer(this);
    scanTimer = new QTimer(this);

    QString enable_debuglog_filename = debuglog_filename_part1;
    enable_debuglog_filename.append(debuglog_filename_part3);
    QFile enable_debuglog_file ( enable_debuglog_filename );
    if ( enable_debuglog_file.exists() ) {
//       kError() << "KCHILDLOCK:" << "start of daemon";
       debugflag = true;
       QString debuglog_filename = debuglog_filename_part1;
       QDateTime currdt = QDateTime::currentDateTime ();
       debuglog_filename.append(currdt.toString ("yyyyMMdd_hhmmss"));
       debuglog_filename.append(debuglog_filename_part3);
       debugfile = fopen ( debuglog_filename.toAscii(), "a+");
       }
    else
       debugflag = false;
    
    if (debugflag) {
       fprintf (debugfile,"\nStarting kchildlock daemon (version %s) ...\n", KCHILDLOCK_VERSION);
       fprintf (debugfile,"date and time is %i.%i.%i. %i:%i:%i\n",
	QDate::currentDate().day(), QDate::currentDate().month(), QDate::currentDate().year(),
	QTime::currentTime().hour(), QTime::currentTime().minute(), QTime::currentTime().second());
       fflush(debugfile);
       }


    connect(startupTimer, SIGNAL(timeout()), this, SLOT(startupTimePassed()));
    startupTimer->start(15 * 1000);

}
    

// Destroy the object
// ==================

KchildlockDaemon::~KchildlockDaemon()
{

}




// after the initial delay begin work
// ==================================
void KchildlockDaemon::startupTimePassed()
{

   bool retvalue;
   int idx;
   
   if (debugflag) {
      fprintf (debugfile,"\nafter the initial timeout...\n");
      fprintf (debugfile,"date and time is %i.%i.%i. %i:%i:%i\n",
	QDate::currentDate().day(), QDate::currentDate().month(), QDate::currentDate().year(),
	QTime::currentTime().hour(), QTime::currentTime().minute(), QTime::currentTime().second());
      fflush (debugfile);
      }

   startupTimer->stop();

   current_user = new CurrentUserState();
   current_user->set_prev_user("");
   current_user->set_user(get_active_user ());
   my_consumption = new Consumption();
   retvalue = my_consumption->exists_record(current_user->get_user());

   if (debugflag) {
      fprintf (debugfile,"filename %s\n", my_consumption->get_consumption_filename().toAscii().constData());
      fflush(debugfile);
      }

// get values from consumed time file
    if ( retvalue )         // record found in file
    {
            my_consumption->read_record();

            if (debugflag) {
               fprintf (debugfile,"consumed times values from file;\n");
               fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->my_cons_record.pcu.weekofyear, my_consumption->my_cons_record.pcu.dayofweek, my_consumption->my_cons_record.pcu.pcusage_daily, my_consumption->my_cons_record.pcu.pcusage_weekly);
               for (idx=0; idx<KCH_NUMOFAPPS; idx++) {
                   fprintf (debugfile,"appPT=%s, appPD=%s, usd=%i, usw=%i\n", my_consumption->my_cons_record.app[idx].appnamePT, my_consumption->my_cons_record.app[idx].appnamePD, my_consumption->my_cons_record.app[idx].appusage_daily, my_consumption->my_cons_record.app[idx].appusage_weekly);
               }
               fflush(debugfile);
            }
            
      }
      else
// set default values, if no record was found in file
      {
         my_consumption->create_record(current_user->get_user());
         if (debugflag) {
            fprintf (debugfile,"User %s gets default consumed times values without file;\n", (current_user->get_user()).toAscii().constData());
            fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->my_cons_record.pcu.weekofyear, my_consumption->my_cons_record.pcu.dayofweek, my_consumption->my_cons_record.pcu.pcusage_daily, my_consumption->my_cons_record.pcu.pcusage_weekly);
            fflush(debugfile);
          }
      }

   if ( my_consumption->my_cons_record.pcu.weekofyear != QDate::currentDate().weekNumber() )
      {
      // its a new week, reset all consumed times values to zero
      my_consumption->read_record();
      my_consumption->set_default_values(current_user->get_user());
      my_consumption->write_record();
      if (debugflag) {
         fprintf (debugfile,"User %s gets default consumed times values for a new week;\n", (current_user->get_user()).toAscii().constData());
         fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->my_cons_record.pcu.weekofyear, my_consumption->my_cons_record.pcu.dayofweek, my_consumption->my_cons_record.pcu.pcusage_daily, my_consumption->my_cons_record.pcu.pcusage_weekly);
         fflush(debugfile);
         }
      }


    myGeneralSettingsKCG = KSharedConfig::openConfig("kchildlockrc")->group("General");
    UserSettings_filename  = get_UserSettings_filename(myGeneralSettingsKCG.readEntry("usernamestringlist"));
    GroupSettings_filename  = get_GroupSettings_filename(myGeneralSettingsKCG.readEntry("groupnamestringlist"));

    if (UserSettings_filename=="" && GroupSettings_filename=="") return;     // do nothing for this user
    if (UserSettings_filename != "") {
       myUserSettingsKCG = KSharedConfig::openConfig(UserSettings_filename)->group("UserGroup");
    }
    if (GroupSettings_filename != "") {
       myGroupSettingsKCG = KSharedConfig::openConfig(GroupSettings_filename)->group("UserGroup");
    }

// get global config settings
    scanint =  myGeneralSettingsKCG.readEntry("scaninterval", "60").toInt();
    if (myGeneralSettingsKCG.readEntry("enable_restrictions", "false") == "false") {
       return;		// do nothing if disabled
    }
    
// create limits object
   current_limits = new KchildlockLimits();
   monitor_user();

    if (myGeneralSettingsKCG.readEntry("enable_systray", "false") == "true") {
      if (!current_limits->get_nolimits()) {
         MainWindow *window = new MainWindow(0, current_limits, my_consumption);
      }
    }
   
  connect(scanTimer, SIGNAL(timeout()), this, SLOT(scanTimePassed()));
  scanTimer->start(1000 * scanint);

}




// after each scanintervall, moitor the system
// ===========================================
void KchildlockDaemon::scanTimePassed()
{
  bool retvalue;
   scanTimer->stop();
   scanTimer->start(1000 * scanint);
   if (debugflag) {
      fprintf (debugfile,"\nafter the repeating timeout...\n");
      fprintf (debugfile,"date and time is %i.%i.%i. %i:%i:%i\n",
	QDate::currentDate().day(), QDate::currentDate().month(), QDate::currentDate().year(),
	QTime::currentTime().hour(), QTime::currentTime().minute(), QTime::currentTime().second());
      fprintf (debugfile,"actual user is %s\n", current_user->get_user().toAscii().constData());
      fflush(debugfile);
      }

   current_user->set_user(get_active_user ());
   if (current_user->get_user() == current_user->get_prev_user()) {
     monitor_user();
   } else {
     if (debugflag) {
        fprintf (debugfile,"User has changed - consumed times values from file;\n");
        fprintf (debugfile,"olduser=%s, newuser=%s\n", (current_user->get_prev_user()).toAscii().constData(), (current_user->get_user()).toAscii().constData());
        fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->my_cons_record.pcu.weekofyear, my_consumption->my_cons_record.pcu.dayofweek, my_consumption->my_cons_record.pcu.pcusage_daily, my_consumption->my_cons_record.pcu.pcusage_weekly);
//        for (idx=0; idx<KCH_NUMOFAPPS; idx++) {
//            fprintf (debugfile,"appPT=%s, appPD=%s, usd=%i, usw=%i\n", my_consumption->my_cons_record.app[idx].appnamePT, my_consumption->my_cons_record.app[idx].appnamePD, my_consumption->my_cons_record.app[idx].appusage_daily, my_consumption->my_cons_record.app[idx].appusage_weekly);
//        }
        fflush(debugfile);
     }
     retvalue = my_consumption->exists_record(current_user->get_user());
//   get values from consumed time file
     if ( retvalue ) {        // record found in file
        my_consumption->read_record();
     }
     else {
// set default values, if no record was found in file
         my_consumption->create_record(current_user->get_user());
         if (debugflag) {
            fprintf (debugfile,"User %s gets default consumed times values without file;\n", (current_user->get_user()).toAscii().constData());
            fprintf (debugfile,"woy=%i. dow=%i. usd=%i, usw=%i\n", my_consumption->my_cons_record.pcu.weekofyear, my_consumption->my_cons_record.pcu.dayofweek, my_consumption->my_cons_record.pcu.pcusage_daily, my_consumption->my_cons_record.pcu.pcusage_weekly);
            fflush(debugfile);
         }
     }
     current_user->set_prev_user(current_user->get_user());
     monitor_user();
   }

}






// get the filename for the User Settings of the active user
// =========================================================
QString KchildlockDaemon::get_UserSettings_filename(QString usernamestringlist)
{
  QStringList  usernames = usernamestringlist.split(",");
  QString retval;
  if (usernames.indexOf(current_user->get_user()) >= 0) {
    retval = "kchildlockrc_U_" + current_user->get_user();
  } else {
    retval = "";
  }
  return(retval);
}


// get the filename for the Group Settings of the active user (first group the user belongs to)
// ============================================================================================
QString KchildlockDaemon::get_GroupSettings_filename(QString groupnamestringlist)
{
   QStringList  groupnames = groupnamestringlist.split(",");
   QStringList myuserlist;
   QString retval = "";
   KUserGroup mygroup;
   bool ismember;

  for ( QStringList::Iterator it = groupnames.begin(); it != groupnames.end(); ++it ) {
     mygroup = KUserGroup(*it);
     myuserlist = mygroup.userNames();
     ismember = FALSE;
     for ( QStringList::Iterator it2 = myuserlist.begin(); it2 != myuserlist.end(); ++it2 ) {
       if ((*it2) == current_user->get_user()) {
         ismember = TRUE;
         break;
       }
     }
     if (ismember) 
        break;
  }
  if (ismember) {
     retval = "kchildlockrc_G_" + mygroup.name();
  }
  return(retval);
}





// functions for the handling of the current user
// ==============================================

CurrentUserState::CurrentUserState()
{
   int idx;
   user = "";
   prev_user = "";
//   disp = "";
   warn_usr_state = KCHILDLOCK_NOWARNSTATE;
   warn_app_state = KCHILDLOCK_NOWARNSTATE;
   for (idx=0; idx<KCH_NUMOFAPPS; idx++) {
       appisactivePT[idx] = FALSE;
       appisactivePD[idx] = FALSE;
   }
}

CurrentUserState::~CurrentUserState()
{
}

QString CurrentUserState::get_user()
{
   return user;
}

QString CurrentUserState::get_prev_user()
{
   return prev_user;
}

//QString CurrentUserState::get_disp()
//{
//   return disp;
//}

void CurrentUserState::set_user(QString newuser)
{
   user = newuser;
}

void CurrentUserState::set_prev_user(QString newprevuser)
{
   prev_user = newprevuser;
}

//void CurrentUserState::set_disp(QString newdisp)
//{
//   disp = newdisp;
//}

void CurrentUserState::set_appisactivePT(int idx, bool value)
{
   appisactivePT[idx] = value;
}

bool CurrentUserState::get_appisactivePT(int idx)
{
   return(appisactivePT[idx]);
}

void CurrentUserState::set_appisactivePD(int idx, bool value)
{
   appisactivePD[idx] = value;
}

bool CurrentUserState::get_appisactivePD(int idx)
{
   return(appisactivePD[idx]);
}


#include "moc_KchildlockDaemon.cpp"
